# -*- coding: utf-8 -*-
import six

string_types = six.string_types

if six.PY3:
    from unittest import mock  # noqa

    def make_str(value, encoding='utf-8', errors='replace'):
        if isinstance(value, bytes):
            return value.decode(encoding=encoding, errors=errors)
        return str(value)

    def prepare_for_write(value):
        return make_str(value)

    def read_file(f):
        content = f.read()
        content_type = content.split('\n')[0]
        content = '\n'.join(content.split('\n')[1:])
        return content_type, content

    def create_response_body(content):
        return six.BytesIO(content.encode(encoding='utf-8', errors='replace'))

else:
    import mock  # noqa
    _text_type = unicode  # noqa

    def make_str(value, encoding='utf-8', errors='replace'):
        if isinstance(value, _text_type):
            return value.encode(encoding=encoding, errors=errors)
        return _text_type(value, errors=errors).encode(encoding=encoding, errors=errors)

    def prepare_for_write(value):
        return make_str(value)

    def read_file(f):
        content = f.read()
        content_type = content.split('\n')[0]
        content = '\n'.join(content.split('\n')[1:])
        return content_type, content

    def create_response_body(content):
        return six.StringIO(content)



# -*- coding: utf-8 -*-
from .base import Mock
from rmoq.backends import FileStorageBackend, RmoqStorageBackend, MemcachedStorageBackend


_mock = Mock()

__all__ = [
    RmoqStorageBackend,
    FileStorageBackend,
    MemcachedStorageBackend,
]
for __attr in (a for a in dir(_mock)):
    __all__.append(__attr)
    globals()[__attr] = getattr(_mock, __attr)



# -*- coding: utf-8 -*-
import os
import re

from . import compat


class RmoqStorageBackend(object):
    """
    Base backend for rmoq backends. All storage backends for rmoq must inherit this
    backend if it is to be used with rmoq.
    """

    def get(self, prefix, url):
        """
        Fetches a request response from storage. Should be overridden by subclasses.

        :param prefix: A prefix that separates containers of request responses in the storage.
        :param url: The url of the request.
        """
        raise NotImplementedError

    def put(self, prefix, url, content, content_type):
        """
        Writes a request response in to storage. Should be overridden by subclasses.

        :param prefix: A prefix that separates containers of request responses in the storage.
        :param url: The url of the request.
        :param content: The content of the request response.
        :param content_type: The content type header of the request response.
        """
        raise NotImplementedError

    @staticmethod
    def _parse(content):
        return (
            compat.make_str(content.split('\n')[0]),
            compat.make_str('\n'.join(content.split('\n')[1:]))
        )

    @staticmethod
    def clean_url(url, replacement='_'):
        """
        Cleans the url for protocol prefix and trailing slash and replaces special characters
        with the given replacement.

        :param url: The url of the request.
        :param replacement: A string that is used to replace special characters.
        """
        cleaned = re.sub(r'/$', '', re.sub(r'https?://', '', url))
        for character in '/ _ ? & : ; %'.split():
            cleaned = cleaned.replace(character, replacement)
        return cleaned


class FileStorageBackend(RmoqStorageBackend):
    """
    A rmoq backend that reads and writes to the local file system.
    This is the default backend.
    """

    def get(self, prefix, url):
        filename = self.get_filename(prefix, url)
        if os.path.exists(filename):
            with open(filename) as f:
                return compat.read_file(f)

    def put(self, prefix, url, content, content_type):
        filename = self.get_filename(prefix, url)

        if not os.path.exists(os.path.dirname(filename)):
            os.makedirs(os.path.dirname(filename))

        with open(filename, mode='w') as f:
            f.writelines([content_type, '\n', compat.prepare_for_write(content)])

    def get_filename(self, prefix, url):
        """
        Creates a file path on the form: current-working-directory/prefix/cleaned-url.txt

        :param prefix: The prefix from the .get() and .put() methods.
        :param url: The url of the request.
        :return: The created path.
        """
        return '{}.txt'.format(os.path.join(os.getcwd(), prefix, self.clean_url(url)))


class MemcachedStorageBackend(RmoqStorageBackend):
    """
    Storage backend for rmoq that uses memcached for storage. Takes a the same arguments
    as python-memcached: a list of servers and options as keyword arguments.
    """

    def __init__(self, servers, **options):
        import memcache

        self.client = memcache.Client(servers=servers, **options)

    def get(self, prefix, url):
        return self._parse(self.client.get(self.create_key(prefix, url)))

    def put(self, prefix, url, content, content_type):
        return self.client.add(
            self.create_key(prefix, url),
            '\n'.join([content_type, content]),
            60 * 60 * 24
        )

    def create_key(self, *parts):
        return ''.join([self.clean_url(part, '') for part in parts])



import codecs
from os import path

from setuptools import setup, find_packages


def read(*parts):
    file_path = path.join(path.dirname(__file__), *parts)
    return codecs.open(file_path, encoding='utf-8').read()

setup(
    name='rmoq',
    version='0.4.1',
    url='http://github.com/relekang/rmoq',
    author='Rolf Erik Lekang',
    author_email='me@rolflekang.com',
    description='A simple request-mocker that will download',
    long_description=read('README.rst'),
    packages=find_packages(exclude='tests'),
    install_requires=[
        'mock==1.0.1',
        'six==1.8.0',
        'requests',
    ],
    license='MIT',
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ]
)



# -*- coding: utf-8 -*-
import ast
import os

import requests
from requests.packages.urllib3 import HTTPResponse

from . import compat
from .backends import FileStorageBackend, RmoqStorageBackend


class Mock(object):
    """
    The mocker class that mocks requests in rmoq. It supports being used in with-statements
    and have a method :method:`activate` that can be used as a decorator on functions or classes.
    """

    def __init__(self, prefix='fixtures', backend=FileStorageBackend()):
        self.prefix = prefix
        self.backend = backend

    def __enter__(self):
        def on_send(session, request, *args, **kwargs):
            return self.on_request(session, request, *args, **kwargs)

        if not self.disabled:
            self.patch = compat.mock.patch('requests.Session.send', on_send)
            self.patch.start()

    def __exit__(self, exc_type, exc_val, exc_tb):
        if not self.disabled:
            self.patch.stop()

    @property
    def disabled(self):
        return ast.literal_eval(os.environ.get('RMOQ_DISABLED', 'False'))

    def activate(self, prefix=None, backend=None):
        """
        A decorator used to activate the mocker.

        :param prefix:
        :param backend: An instance of a storage backend.
        """
        if isinstance(prefix, compat.string_types):
            self.prefix = prefix

        if isinstance(backend, RmoqStorageBackend):
            self.backend = backend

        def activate(func):
            if isinstance(func, type):
                return self._decorate_class(func)

            def wrapper(*args, **kwargs):
                with self:
                    return func(*args, **kwargs)

            return wrapper

        return activate

    def _decorate_class(self, cls):
        for attr in cls.__dict__:
            if callable(getattr(cls, attr)):
                setattr(cls, attr, self.activate()(getattr(cls, attr)))
        return cls

    def on_request(self, session, request, *args, **kwargs):

        content = self.backend.get(self.prefix, request.url)
        if content is not None:
            response = HTTPResponse(
                status=200,
                body=compat.create_response_body(content[1]),
                preload_content=False,
                headers={'Content-Type': content[0]}
            )
            adapter = session.get_adapter(request.url)
            response = adapter.build_response(request, response)
        else:
            self.patch.stop()
            response = requests.get(request.url)
            self.patch.start()
            self.backend.put(
                self.prefix,
                request.url,
                response.text,
                response.headers['Content-Type']
            )

        return response
